@@ -4,6 +4,7 @@ from pathlib import Path
 from typing import Any, Dict, List, Optional, cast
 
 from pycliarr.api.base_api import BaseCliApi, json_data, json_dict, json_list
+from pycliarr.api.exceptions import CliArrError
 
 log = logging.getLogger(__name__)
 
@@ -35,6 +36,18 @@ class BaseCliMediaApi(BaseCliApi):
     api_url_tag = f"{api_url_base}/tag"
     api_url_exclusions = f"{api_url_base}/importlistexclusion"
 
+    def __init__(self, *args, default_root_folder_id = 0, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._default_root_folder_id = default_root_folder_id
+
+    @property
+    def default_root_folder_id(self) -> int:
+        return self._default_root_folder_id
+
+    @default_root_folder_id.setter
+    def default_root_folder_id(self, value: int):
+        self._default_root_folder_id =  value
+
     def get_calendar(self, start_date: Optional[datetime] = None, end_date: Optional[datetime] = None) -> json_data:
         """Retrieve info about when items were/will be downloaded.
 
@@ -123,7 +136,7 @@ class BaseCliMediaApi(BaseCliApi):
         return self.request_get(self.api_url_itemlookup, url_params=url_params)
 
     def add_item(self, json_data: json_data) -> json_data:
-        """addMovie adds a new movie to collection
+        """Adds a new item to collection
 
         Args:
             json_data: Dict representation of the item to add
@@ -147,6 +160,16 @@ class BaseCliMediaApi(BaseCliApi):
         url_path = f"{self.api_url_item}/{item_id}"
         return self.request_delete(url_path, data)
 
+    def edit_item(self, json_data: json_data, url_params: Optional[Dict[str, Any]] = None) -> json_data:
+        """Edit an item from the collection
+
+        Args:
+            json_data: Dict representation of the item to add
+        Returns:
+            json response
+        """
+        return self.request_put(self.api_url_item, json_data=json_data, url_params=url_params)
+
     def get_system_status(self) -> json_data:
         """Return the System Status as json"""
         return self.request_get(self.api_url_systemstatus)
@@ -159,9 +182,41 @@ class BaseCliMediaApi(BaseCliApi):
         """Return the quality profiles"""
         return cast(json_list, self.request_get(self.api_url_language_profile))
 
-    def get_queue(self) -> json_data:
-        """Get queue info (downloading/completed, ok/warning) as json"""
-        return self.request_get(self.api_url_queue)
+    def _get_queue(
+        self,
+        page: int = 1,
+        sort_key: str = "progress",
+        page_size: int = 20,
+        sort_dir: str = "ascending",
+        options: Dict[str, Any] = {},
+    ) -> json_data:
+        """Get queue info (downloading/completed, ok/warning) as json
+
+        Args:
+            page (int) - 1-indexed (1 default)
+            sort_key (string) - title or date
+            page_size (int) - Default: 10
+            sort_dir (string) - asc or desc - Default: asc
+            options (Dict[str, Any]={}): Optional additional options
+        """
+        data = {
+            "page": page,
+            "pageSize": page_size,
+            "sortKey": sort_key,
+            "sortDirection": sort_dir,
+        }
+        data.update(options)
+        return self.request_get(self.api_url_queue, url_params=data)
+
+    def get_queue(
+        self,
+        page: int = 1,
+        sort_key: str = "progress",
+        page_size: int = 20,
+        sort_dir: str = "ascending",
+        include_unknown: bool = True,
+    ) -> json_data:
+        return self._get_queue(page, sort_key, page_size, sort_dir)
 
     def delete_queue(self, item_id: int, blacklist: Optional[bool] = None) -> json_data:
         """Delete an item from the queue and download client. Optionally blacklist item after deletion.
@@ -254,16 +309,22 @@ class BaseCliMediaApi(BaseCliApi):
         """Build an item folder path using the root folder specified.
         Args:
             title (str): Title to add to root path. All invalid characters are removed
-            root_folder_id (int): Id of the root folder (can be retrieved with get_root_folder())
-                If the id is not found or not specified, the first root folder in the list is used.
+            root_folder_id (int): Id of the root folder (can be retrieved with get_root_folder()).
+            If the id is not found or not specified, the default root folder id will be used.
+            If 0, the first root folder in the list is used.
 
         Returns: Full path of the serie in the format <root path>/<serie name>
         """
         root_paths = self.get_root_folder()
-        root_path = root_paths[0]
+        selected_root_folder_id = root_folder_id or self.default_root_folder_id
+        print(f"{selected_root_folder_id} // {root_folder_id} // {self.default_root_folder_id}")
+        root_path = root_paths[0] if not selected_root_folder_id else None
         for path in root_paths:
-            if path["id"] == root_folder_id:
+            if path["id"] == int(selected_root_folder_id):
                 root_path = path
+
+        if not root_path:
+            raise CliArrError(f"Invalid root folder Id: {selected_root_folder_id}")
         return Path(root_path["path"]) / self.to_path(title)
 
     def get_blocklist(
