@@ -13,6 +13,7 @@ class RadarrMovieItem(BaseCliApiItem):
         """Define the model of items represented by this class."""
         return {
             "title": "",
+            "originalTitle": "",
             "sortTitle": "",
             "sizeOnDisk": 0,
             "overview": "",
@@ -28,7 +29,6 @@ class RadarrMovieItem(BaseCliApiItem):
             "studio": "",
             "path": "",
             "rootFolderPath": "",
-            "profileId": 0,
             "monitored": True,
             "minimumAvailability": "",
             "isAvailable": "",
@@ -44,8 +44,9 @@ class RadarrMovieItem(BaseCliApiItem):
             "added": None,
             "ratings": {},
             "collection": {},
-            "alternativeTitles": [],
+            "alternateTitles": [],
             "qualityProfileId": 0,
+            "secondaryYearSourceId": 0,
             "id": 0,
         }
 
@@ -55,6 +56,7 @@ class RadarrCli(BaseCliMediaApi):
 
     Radarr API reference:
         https://github.com/Radarr/Radarr/wiki/API
+        https://pub.dev/packages/radarr
 
     Note:
         Not all commands are implemented.
@@ -137,6 +139,7 @@ class RadarrCli(BaseCliMediaApi):
         monitored: bool = True,
         search: bool = True,
         path: Optional[str] = None,
+        root_id: Optional[int] = 0,
     ) -> json_data:
         """addMovie adds a new movie to collection.
 
@@ -151,7 +154,8 @@ class RadarrCli(BaseCliMediaApi):
             monitored (bool): Whether to monitor the movie. Default is True
             search (bool): Whether to search for the movie once added. Default is True
             path (Optional[str]): Specify the path awhere the movie should be stored.
-                Default is root/<movie name> (<movie year>).
+                Default is root[0]/<movie name> (<movie year>).
+            root_id (Optional[int]): Specify the root folder to use. Ignored if a path is specified. Default is root[0].
         Returns:
             json response
         """
@@ -163,8 +167,7 @@ class RadarrCli(BaseCliMediaApi):
             raise RadarrCliError("Error, invalid parameters or invalid tmdb/imdb id")
 
         # Prepare movie info for adding
-        movie_info.path = path or str(self.build_movie_path(movie_info))
-        movie_info.profileId = quality
+        movie_info.path = path or str(self.build_movie_path(movie_info, root_folder_id=root_id))
         movie_info.qualityProfileId = quality
         movie_info.monitored = monitored
         movie_info.add_attribute("addOptions", {"searchForMovie": search})
@@ -180,7 +183,9 @@ class RadarrCli(BaseCliMediaApi):
 
         Returns: Full path of the serie in the format <root path>/<movie name> (<movie year>)
         """
-        return self.build_item_path(movie_info.title + (f" ({movie_info.year})" if movie_info.year else ""))
+        return self.build_item_path(
+            movie_info.title + (f" ({movie_info.year})" if movie_info.year else ""), root_folder_id
+        )
 
     def delete_movie(self, movie_id: int, delete_files: bool = True, add_exclusion: bool = False) -> json_data:
         """Delete the movie with the given ID
@@ -195,6 +200,24 @@ class RadarrCli(BaseCliMediaApi):
         options = {"addImportExclusion": add_exclusion} if add_exclusion else {}
         return self.delete_item(movie_id, delete_files, options)
 
+    def edit_movie(
+        self,
+        movie_info: RadarrMovieItem,
+        move_files: bool = False,
+    ) -> json_data:
+        """Edit a movie from the collection.
+
+        The movie description movie_info must be specified, usually by getting the information from get_movie()
+
+        Args:
+            movie_info (Optional[RadarrMovieItem]): Description of the movie to edit
+            move_files (bool): Whether to move files after edition. Default is False
+        Returns:
+            json response
+        """
+
+        return self.edit_item(json_data=movie_info.to_dict())  # , url_params={"moveFiles": False})
+
     def refresh_movie(self, movie_id: Optional[int] = None) -> json_data:
         """Refresh movie information  and rescan disk.
 
@@ -232,3 +255,31 @@ class RadarrCli(BaseCliMediaApi):
         return self.request_post(
             self.api_url_exclusions, json_data={"movieTitle": title, "tmdbId": tmdb_id, "movieYear": year}
         )
+
+    def missing_movies_search(self) -> json_data:
+        """Search for missing movies.
+        Returns:
+            json response
+        """
+        return self._sendCommand({"name": "MissingMoviesSearch"})
+
+    def get_queue(
+        self,
+        page: int = 1,
+        sort_key: str = "progress",
+        page_size: int = 20,
+        sort_dir: str = "ascending",
+        include_unknown: bool = True,
+    ) -> json_data:
+        """Get queue info (downloading/completed, ok/warning) as json
+
+        Args:
+            page (int) - 1-indexed (1 default)
+            sort_key (string) - title or date
+            page_size (int) - Default: 10
+            sort_dir (string) - asc or desc - Default: asc
+            options (Dict[str, Any]={}): Optional additional options
+        """
+        return self._get_queue(
+            page, sort_key, page_size, sort_dir, options={"includeUnknownMovieItems": include_unknown}
+        )
