@@ -59,6 +59,7 @@ class SonarrCli(BaseCliMediaApi):
 
     API reference:
         https://github.com/Sonarr/Sonarr/wiki/API
+        https://pub.dev/packages/sonarr
 
     Note:
         Not all commands are implemented.
@@ -143,6 +144,7 @@ class SonarrCli(BaseCliMediaApi):
         search: bool = True,
         season_folder: bool = True,
         path: Optional[str] = None,
+        root_id: Optional[int] = 0,
         language: int = 1,
     ) -> json_data:
         """addMovie adds a new serie to collection.
@@ -159,6 +161,7 @@ class SonarrCli(BaseCliMediaApi):
             search (bool): Whether to search for the serie once added. Default is True
             season_folder (bool): If True (default), create a folder for each season.
             path (Optional[str]): Specify the path awhere the movie should be stored. Default is root/<serie name>.
+            root_id (Optional[int]): Specify the root folder to use. Ignored if a path is specified. Default is root[0].
             language (int): Specify the language to use. Default is the first enabled (1)
 
         Returns:
@@ -177,7 +180,7 @@ class SonarrCli(BaseCliMediaApi):
             raise SonarrCliError("Error, invalid parameters or invalid tvdb id")
 
         # Prepare serie info for adding
-        serie_info.path = path or str(self.build_serie_path(serie_info))
+        serie_info.path = path or str(self.build_serie_path(serie_info, root_folder_id=root_id))
         serie_info.profileId = quality
         serie_info.qualityProfileId = quality
         serie_info.languageProfileId = language
@@ -202,12 +205,12 @@ class SonarrCli(BaseCliMediaApi):
         """Build a serie folder path using the root folder specified.
         Args:
             serie_info (SonarrSerieItem) Item for which to build the path
-            root_folder_id (int): Id of the root folder (can be retrieved with get_root_folder())
-                If the id is not found or not specified, the first root folder in the list is used.
+            root_folder_id (int): Id of the root folder (can be retrieved with get_root_folder()).
+            If the id is not found or not specified, the first root folder in the list is used.
 
         Returns: Full path of the serie in the format <root path>/<serie name>
         """
-        return self.build_item_path(serie_info.title)
+        return self.build_item_path(serie_info.title, root_folder_id)
 
     def delete_serie(self, serie_id: int, delete_files: bool = True, add_exclusion: bool = False) -> json_data:
         """Delete the serie with the given ID
@@ -222,6 +225,19 @@ class SonarrCli(BaseCliMediaApi):
         options = {"addImportListExclusion": add_exclusion} if add_exclusion else {}
         return self.delete_item(serie_id, delete_files, options)
 
+    def edit_serie(self, serie_info: SonarrSerieItem) -> json_data:
+        """Edit a serie from the collection.
+
+        The serie description movie_info must be specified, usually by getting the information from get_serie()
+
+        Args:
+            serie_info (Optional[RadarrMovieItem]): Description of the movie to edit
+        Returns:
+            json response
+        """
+
+        return self.edit_item(json_data=serie_info.to_dict())
+
     def refresh_serie(self, serie_id: Optional[int] = None) -> json_data:
         """Refresh serie information  and rescan disk.
 
@@ -305,3 +321,31 @@ class SonarrCli(BaseCliMediaApi):
             json response
         """
         return self.request_post(self.api_url_exclusions, json_data={"title": title, "tvdbId": tvdb_id})
+
+    def missing_episodes_search(self) -> json_data:
+        """Search for missing episodes.
+        Returns:
+            json response
+        """
+        return self._sendCommand({"name": "missingEpisodeSearch"})
+
+    def get_queue(
+        self,
+        page: int = 1,
+        sort_key: str = "progress",
+        page_size: int = 20,
+        sort_dir: str = "ascending",
+        include_unknown: bool = True,
+    ) -> json_data:
+        """Get queue info (downloading/completed, ok/warning) as json
+
+        Args:
+            page (int) - 1-indexed (1 default)
+            sort_key (string) - title or date
+            page_size (int) - Default: 10
+            sort_dir (string) - asc or desc - Default: asc
+            options (Dict[str, Any]={}): Optional additional options
+        """
+        return self._get_queue(
+            page, sort_key, page_size, sort_dir, options={"includeUnknownSeriesItems": include_unknown}
+        )
