@@ -1,10 +1,12 @@
 import datetime
 import json
 from argparse import ArgumentParser, Namespace, _SubParsersAction
+from curses.ascii import isdigit
 from pprint import pformat
 from typing import Any, List, Optional, Union, no_type_check
 
-from pycliarr.api import base_api, base_media, radarr, sonarr
+from pycliarr.api import base_api, base_media, exceptions, radarr, sonarr
+from pycliarr.cli.utils import size_to_str
 
 
 class CliCommand:
@@ -69,7 +71,7 @@ def select_profile(cli: base_media.BaseCliMediaApi) -> int:
     if profile_id.isdigit():
         return int(profile_id)
     else:
-        raise Exception("Invalid profile selection: {}")
+        raise Exception("Invalid profile selection: {profile_id}")
 
 
 def select_language_profile(cli: base_media.BaseCliMediaApi) -> int:
@@ -80,7 +82,7 @@ def select_language_profile(cli: base_media.BaseCliMediaApi) -> int:
     if profile_id.isdigit():
         return int(profile_id)
     else:
-        raise Exception("Invalid profile selection: {}")
+        raise Exception("Invalid profile selection: {profile_id}")
 
 
 def select_item(
@@ -100,6 +102,40 @@ def select_item(
         raise Exception("Invalid selection: {}")
 
 
+def print_root_folder(cli: base_media.BaseCliMediaApi) -> int:
+    res = cli.get_root_folder()
+    print("Id  Free       Path")
+    for root_folder in res:
+        print(f"{root_folder['id']:<3} {size_to_str(root_folder['freeSpace']):<10} {root_folder['path']}")
+
+
+def select_root_folder(cli: base_media.BaseCliMediaApi) -> int:
+    print_root_folder(cli)
+    root_folder_id = input(f"Root folder to use (Id):")
+    if root_folder_id.isdigit():
+        return int(root_folder_id)
+    else:
+        raise Exception("Invalid root folder selection, must be the folder id: {root_folder_id}")
+
+
+def root_folder_id_from_arg(cli: base_media.BaseCliMediaApi, root_arg: str):
+    if root_arg:
+        if root_arg == "auto":
+            # Interactive selection
+            return select_root_folder(cli)
+        elif root_arg.isdigit():
+            # Id specified directly
+            return int(root_arg)
+        else:
+            # Match the path with registered roots to find id
+            res = cli.get_root_folder()
+            for root_path in res:
+                if root_path["path"] == root_arg:
+                    return int(root_path["id"])
+            raise exceptions.CliArrError(f"No root folder with path '{root_arg}'")
+    return 0
+
+
 class CliGetProfilesCommand(CliCommand):
     name = "profiles"
     description = "Get list of quality profiles"
@@ -137,9 +173,18 @@ class CliGetQueueCommand(CliCommand):
     name = "queue"
     description = "Get current downloading queue"
 
+    def configure_args(self, cmd_subparser: _SubParsersAction) -> ArgumentParser:
+        cmd_parser = super().configure_args(cmd_subparser)
+        cmd_parser.add_argument("--page", help="Page to get", type=int, default=1)
+        cmd_parser.add_argument("--sort-key", help="Sort key", default="progress")
+        cmd_parser.add_argument("--page-size", help="Page size", type=int, default=20)
+        cmd_parser.add_argument("--sort-dir", help="Sort direction", default="ascending")
+        cmd_parser.add_argument("--exclude-unknown", help="Exclude unknown items", action="store_true", default=False)
+        return cmd_parser
+
     def run(self, cli: base_media.BaseCliMediaApi, args: Namespace) -> None:
         super().run(cli, args)
-        res = cli.get_queue()
+        res = cli.get_queue(args.page, args.sort_key, args.page_size, args.sort_dir, not args.exclude_unknown)
         print(f"{pformat(res)}\n")
 
 
@@ -443,6 +488,15 @@ class CliDeleteExclusionCommand(CliCommand):
         print(f"{pformat(res)}\n")
 
 
+class CliRootFoldersCommand(CliCommand):
+    name = "root-folders"
+    description = "Get root folder list"
+
+    def run(self, cli: base_media.BaseCliMediaApi, args: Namespace) -> None:
+        super().run(cli, args)
+        print_root_folder(cli)
+
+
 ##############################################
 ########## radarr specific commands ##########
 ##############################################
@@ -455,12 +509,20 @@ class CliGetMovieCommand(CliCommand):
     def configure_args(self, cmd_subparser: _SubParsersAction) -> ArgumentParser:
         cmd_parser = super().configure_args(cmd_subparser)
         cmd_parser.add_argument("--mid", "-i", help="ID of the movie to get info on", type=int, default=None)
+        cmd_parser.add_argument("--json", "-j", action="store_true", help="Print data as json", default=False)
         return cmd_parser
 
     def run(self, cli: radarr.RadarrCli, args: Namespace) -> None:
         super().run(cli, args)
         res = cli.get_movie(args.mid)
-        print(f"Movie info:\n{res}")
+        if args.json:
+            if isinstance(res, list):
+                json_objs = [item.to_json() for item in res]
+                print(f"[{','.join(json_objs)}]")
+            else:
+                print(f"{res.to_json()}")
+        else:
+            print(res)
 
 
 class CliDeleteMovieCommand(CliCommand):
@@ -481,7 +543,7 @@ class CliDeleteMovieCommand(CliCommand):
     def run(self, cli: radarr.RadarrCli, args: Namespace) -> None:
         super().run(cli, args)
         res = cli.delete_movie(args.mid, delete_files=args.delfiles, add_exclusion=args.exclude)
-        print(f"Result:\n{res}")
+        print(res)
 
 
 class CliGetRefreshMovieCommand(CliCommand):
@@ -496,7 +558,7 @@ class CliGetRefreshMovieCommand(CliCommand):
     def run(self, cli: radarr.RadarrCli, args: Namespace) -> None:
         super().run(cli, args)
         res = cli.refresh_movie(args.mid)
-        print(f"Result: {json.dumps(res)}\n")
+        print(res)
 
 
 class CliGetRescanMovieCommand(CliCommand):
@@ -511,7 +573,7 @@ class CliGetRescanMovieCommand(CliCommand):
     def run(self, cli: radarr.RadarrCli, args: Namespace) -> None:
         super().run(cli, args)
         res = cli.rescan_movie(args.mid)
-        print(f"Result: {json.dumps(res)}\n")
+        print(res)
 
 
 class CliAddMovieCommand(CliCommand):
@@ -529,6 +591,12 @@ class CliAddMovieCommand(CliCommand):
         )
         cmd_parser.add_argument("--quality", "-q", help="Quality profile to use", type=int, default=None)
         cmd_parser.add_argument("--path", help="Full path where the serie should be stored", type=str, default=None)
+        cmd_parser.add_argument(
+            "--root-folder", '-r',
+            help="Root folder id or path, or 'auto' to select it interactively. Ignored if --path is set",
+            type=str,
+            default=None,
+        )
         return cmd_parser
 
     def run(self, cli: radarr.RadarrCli, args: Namespace) -> None:
@@ -542,6 +610,7 @@ class CliAddMovieCommand(CliCommand):
         # If no quality profile specified, list them qnd prompt for choice
         if not args.quality:
             args.quality = select_profile(cli)
+        root_id = root_folder_id_from_arg(cli, args.root_folder)
 
         res = cli.add_movie(
             quality=args.quality,
@@ -549,8 +618,32 @@ class CliAddMovieCommand(CliCommand):
             imdb_id=args.imdb,
             movie_info=movie_info,  # type: ignore
             path=args.path,
+            root_id=root_id,
         )
-        print(f"Result:\n{res}")
+        print(f"{json.dumps(res)}")
+
+
+class CliEditMovieCommand(CliCommand):
+    name = "edit"
+    description = "Push an updated item to the movie library"
+
+    def configure_args(self, cmd_subparser: _SubParsersAction) -> ArgumentParser:
+        cmd_parser = super().configure_args(cmd_subparser)
+        search_group = cmd_parser.add_mutually_exclusive_group()
+        search_group.required = True
+        search_group.add_argument("--json", "-j", help="json data")
+        search_group.add_argument("--file", "-f", help="json file")
+        return cmd_parser
+
+    def run(self, cli: radarr.RadarrCli, args: Namespace) -> None:
+        super().run(cli, args)
+        json_data = args.json
+        if not json_data and args.file:
+            with open(args.file, "r") as f:
+                json_data = f.read()
+        info = radarr.RadarrMovieItem.from_json(json_data)
+        res = cli.edit_movie(info)
+        print(f"{json.dumps(res)}")
 
 
 class CliCreateRadarrExclusionCommand(CliCommand):
@@ -570,9 +663,21 @@ class CliCreateRadarrExclusionCommand(CliCommand):
         print(f"{pformat(res)}\n")
 
 
+class CliSearchMissingMovies(CliCommand):
+    name = "search-missing"
+    description = "Search missing movies"
+
+    def run(self, cli: radarr.RadarrCli, args: Namespace) -> None:
+        super().run(cli, args)
+        res = cli.missing_movies_search()
+        print(f"{json.dumps(res)}")
+
+
 ##############################################
 ########## sonarr specific commands ##########
 ##############################################
+
+
 class CliGetSerieCommand(CliCommand):
     name = "get"
     description = "Get info on a of serie"
@@ -580,12 +685,20 @@ class CliGetSerieCommand(CliCommand):
     def configure_args(self, cmd_subparser: _SubParsersAction) -> ArgumentParser:
         cmd_parser = super().configure_args(cmd_subparser)
         cmd_parser.add_argument("--sid", "-i", help="ID of the serie to get info on", type=int, default=None)
+        cmd_parser.add_argument("--json", "-j", action="store_true", help="Print data as json", default=False)
         return cmd_parser
 
     def run(self, cli: sonarr.SonarrCli, args: Namespace) -> None:
         super().run(cli, args)
         res = cli.get_serie(args.sid)
-        print(f"Serie info:\n{res}")
+        if args.json:
+            if isinstance(res, list):
+                json_objs = [item.to_json() for item in res]
+                print(f"[{','.join(json_objs)}]")
+            else:
+                print(f"{res.to_json()}")
+        else:
+            print(res)
 
 
 class CliDeleteSerieCommand(CliCommand):
@@ -662,6 +775,12 @@ class CliAddSerieCommand(CliCommand):
             action="store_true",
         )
         cmd_parser.add_argument("--path", help="Full path where the serie should be stored", type=str, default=None)
+        cmd_parser.add_argument(
+            "--root-folder", '-r',
+            help="Root folder id or path, or 'auto' to select it interactively. Ignored if --path is set",
+            type=str,
+            default=None,
+        )
         return cmd_parser
 
     def run(self, cli: sonarr.SonarrCli, args: Namespace) -> None:
@@ -677,6 +796,7 @@ class CliAddSerieCommand(CliCommand):
             args.quality = select_profile(cli)
         if not args.language:
             args.language = select_language_profile(cli)
+        root_id = root_folder_id_from_arg(cli, args.root_folder)
 
         # Get the optional season list
         seasons_str = args.seasons.replace(" ", "").split(",") if args.seasons else []
@@ -692,6 +812,7 @@ class CliAddSerieCommand(CliCommand):
             monitored_seasons=seasons,
             season_folder=args.season_folders,
             path=args.path,
+            root_id=root_id,
             language=args.language,
         )
         print(f"Result:\n{res}")
@@ -760,6 +881,16 @@ class CliCreateSonarrExclusionCommand(CliCommand):
         print(f"{pformat(res)}\n")
 
 
+class CliSearchMissingEpisodes(CliCommand):
+    name = "search-missing"
+    description = "Search missing episods"
+
+    def run(self, cli: sonarr.SonarrCli, args: Namespace) -> None:
+        super().run(cli, args)
+        res = cli.missing_episodes_search()
+        print(f"Result: {json.dumps(res)}\n")
+
+
 #####################################################
 ########## Clients and commands definition ##########
 #####################################################
@@ -798,6 +929,8 @@ CLI_LIST: List[CliApiCommand] = [
             CliGetExclusionCommand(),
             CliDeleteExclusionCommand(),
             CliCreateSonarrExclusionCommand(),
+            CliSearchMissingEpisodes(),
+            CliRootFoldersCommand(),
         ],
     ),
     CliApiCommand(
@@ -807,6 +940,7 @@ CLI_LIST: List[CliApiCommand] = [
             CliGetMovieCommand(),
             CliDeleteMovieCommand(),
             CliAddMovieCommand(),
+            CliEditMovieCommand(),
             CliGetRefreshMovieCommand(),
             CliGetRescanMovieCommand(),
             CliGetProfilesCommand(),
@@ -831,6 +965,8 @@ CLI_LIST: List[CliApiCommand] = [
             CliGetExclusionCommand(),
             CliDeleteExclusionCommand(),
             CliCreateRadarrExclusionCommand(),
+            CliSearchMissingMovies(),
+            CliRootFoldersCommand(),
         ],
     ),
 ]
